@@ -61,13 +61,16 @@ scholarLogoTemplate = """
 <svg x="{0}" y="{1}" width="{2}" height="{2}" viewBox="0 0 512 512"><path fill="#4285f4" d="M256 411.12L0 202.667 256 0z"/><path fill="#356ac3" d="M256 411.12l256-208.453L256 0z"/><circle fill="#a0c3ff" cx="256" cy="362.667" r="149.333"/><path fill="#76a7fa" d="M121.037 298.667c23.968-50.453 75.392-85.334 134.963-85.334s110.995 34.881 134.963 85.334H121.037z"/></svg>
 """
 
-def generateBibliometricsImage(metrics, colors, titleText) :
+def generateBibliometricsImage(metrics, colors, titleText, stats) :
     """Generates the bibliometrics image as an SVG.
 
     Keyword arguments:
     metrics - dictionary with the stats
     colors - dictionary with colors
+    titleText - text for the title of the svg
+    stats - a list of the keys of the metrics to include in the order to include them
     """
+    stats = [ key for key in stats if key in metrics ]
     titleSize = 18
     titleLineHeight = 2 * titleSize + 1
     textSize = 14
@@ -80,18 +83,18 @@ def generateBibliometricsImage(metrics, colors, titleText) :
     drop = round(textSize * 12.5 / 14, 1)
     scholarLogoDimensions = 32
 
-    stats = [
-        ("Total citations", "total"),
-        ("Five-year citations", "fiveYear"),
-        ("Most-cited paper", "most"),
-        ("h-index", "h"),
-        ("g-index", "g"),
-        ("e-index", "e"),
-        ("i10-index", "i10"),
-        ("i100-index", "i100"),
-        ("i1000-index", "i1000"),
-        ("i10000-index", "i10000")
-    ]
+    stat_labels = {
+        "total" : "Total citations",
+        "fiveYear" : "Five-year citations",
+        "most" : "Most-cited paper",
+        "h" : "h-index",
+        "g" : "g-index",
+        "i10" : "i10-index",
+        "i100" : "i100-index",
+        "i1000" : "i1000-index",
+        "i10000" : "i10000-index",
+        "e" : "e-index",
+    }
 
     lastUpdatedText = "Last updated: " + date.today().strftime("%d %B %Y")
     lastUpdatedLength = calculateTextLength(lastUpdatedText, smallSize, True, 600)
@@ -99,7 +102,8 @@ def generateBibliometricsImage(metrics, colors, titleText) :
     titleLength = round(calculateTextLength110Weighted(titleText, 600))
     minWidth = calculateTextLength(titleText, titleSize, True, 600) + 4*margin + 2*scholarLogoDimensions
     minWidth = max(minWidth, lastUpdatedLength + 2*margin)
-    for label, key in stats :
+    for key in stats :
+        label = stat_labels[key]
         minWidth = max(minWidth, 2 * calculateTextLength(label, textSize, True, 600) + 2*margin)
     minWidth = math.ceil(minWidth)
 
@@ -118,25 +122,25 @@ def generateBibliometricsImage(metrics, colors, titleText) :
     scale = round(0.75 * textSize / 110, 3)
 
     formattedStats = []
-    for label, key in stats :
-        if key in metrics :
-            offset += lineHeight
-            minHeight += lineHeight
-            data = str(metrics[key])
-            dataWidthPreScale = round(calculateTextLength110Weighted(data, 600))
-            entry = metricTemplate.format(
-                margin,
-                offset,
-                scale,
-                round(calculateTextLength110Weighted(label, 600)),
-                0,
-                round(drop/scale),
-                label,
-                dataWidthPreScale,
-                round((minWidth - 2*margin)/scale) - dataWidthPreScale,   #round(minWidth/2/scale),
-                data
-            )
-            formattedStats.append(entry)
+    for key in stats :
+        label = stat_labels[key]
+        offset += lineHeight
+        minHeight += lineHeight
+        data = str(metrics[key])
+        dataWidthPreScale = round(calculateTextLength110Weighted(data, 600))
+        entry = metricTemplate.format(
+            margin,
+            offset,
+            scale,
+            round(calculateTextLength110Weighted(label, 600)),
+            0,
+            round(drop/scale),
+            label,
+            dataWidthPreScale,
+            round((minWidth - 2*margin)/scale) - dataWidthPreScale,   #round(minWidth/2/scale),
+            data
+        )
+        formattedStats.append(entry)
 
     scale = round(0.75 * smallSize / 110, 3)
 
@@ -230,7 +234,8 @@ def parseBibliometrics(page) :
         return metrics
     i10 = page[startStat+1:endStat]
     metrics["i10"] = int(i10.strip())
-    g, most, i100, i1000, i10000, e = calculate_additional(page, metrics["h"])
+    cites_list = parse_cites_per_pub(page)
+    g, most, i100, i1000, i10000, e = calculate_additional(cites_list, metrics["h"])
     if g > 0 and g < 100 :
         metrics["g"] = g
     if most > 0 :
@@ -245,39 +250,49 @@ def parseBibliometrics(page) :
         metrics["e"] = "{0:.2f}".format(e)
     return metrics
 
-def calculate_additional(page, h) :
+def calculate_additional(cites_list, h) :
     """Calculates the g-index, e-index, i100-index, i1000-index, i10000-index,
     and the most cited paper.
     
     Keyword arguments:
-    page - The user profile page.
+    cites_list - A list of citation counts for each publication.
     h - The h-index, necessary to compute e-index.
     """
-    citesList = parseDataForG(page)
-    if len(citesList) > 0 :
-        citesList.sort(reverse=True)
-        most = citesList[0]
-        i100 = sum(1 for x in citesList if x >= 100)
-        i1000 = sum(1 for x in citesList if x >= 1000)
-        i10000 = sum(1 for x in citesList if x >= 10000)
-        e = calculate_e_index(citesList, h) if h <= 100 else 0
-        for i in range(1, len(citesList)) :
-            citesList[i] = citesList[i] + citesList[i-1]
-        citesList = [ (i+1, x) for i, x in enumerate(citesList) ]
-        return max(y for y, x in citesList if x >= y*y), most, i100, i1000, i10000, e
+    if len(cites_list) > 0 :
+        cites_list.sort(reverse=True)
+        most = cites_list[0]
+        i100 = sum(1 for x in cites_list if x >= 100)
+        i1000 = sum(1 for x in cites_list if x >= 1000)
+        i10000 = sum(1 for x in cites_list if x >= 10000)
+        e = calculate_e_index(cites_list, h) if h <= 100 else 0
+        g = calculate_g_index(cites_list)
+        return g, most, i100, i1000, i10000, e
     return 0, 0, 0, 0, 0, 0
 
-def calculate_e_index(citesList, h) :
+def calculate_g_index(cites_list) :
+    """Calculates the g-index.
+
+    Keyword arguments:
+    cites_list - List of citations of papers in decreasing order.
+    """
+    rolling_sum = [ cites_list[0] ]
+    for i in range(1, len(cites_list)) :
+        rolling_sum.append(cites_list[i] + rolling_sum[i-1])
+    rolling_sum = [ (i+1, x) for i, x in enumerate(rolling_sum) ]
+    return max(y for y, x in rolling_sum if x >= y*y)
+
+def calculate_e_index(cites_list, h) :
     """Calculates the e-index.
 
     Keyword arguments:
-    citesList - List of citations of papers in decreasing order.
+    cites_list - List of citations of papers in decreasing order.
     h - The h-index.
     """
-    return math.sqrt(sum(citesList[i] for i in range(h)) - h*h)
+    return math.sqrt(sum(cites_list[i] for i in range(h)) - h*h)
     
-def parseDataForG(page) :
-    """Parses the cites per publication for calculating g-index
+def parse_cites_per_pub(page) :
+    """Parses the cites per publication for calculating g-index,
+    e-index, i100-index, etc.
 
     Keyword arguments:
     page - The user profile page
@@ -406,6 +421,8 @@ def validateMetrics(metrics) :
         print("ERROR: Failed to parse i10-index.")
     if "g" not in metrics :
         print("WARNING: Failed to parse data needed to compute g-index.")
+    if "e" not in metrics :
+        print("WARNING: Failed to parse data needed to compute e-index.")
     if "most" not in metrics :
         print("WARNING: Failed to parse data needed to compute most-cited paper.")
     if not valid :
@@ -434,6 +451,24 @@ def main() :
     metrics = parseBibliometrics(page)
     validateMetrics(metrics)
 
+    # default metrics in default order
+    stats = [
+        "total",
+        "fiveYear",
+        "most",
+        "h",
+        "g",
+        "i10",
+        "i100",
+        "i1000",
+        "i10000",
+        "e"
+    ]
+
+    # check if user-specified metrics order for all SVGs
+    if "include" in configuration :
+        stats = configuration["include"]
+
     if previousMetrics != metrics :
         if "jsonOutputFile" in configuration :
             outputJSON(configuration["jsonOutputFile"], metrics)
@@ -442,6 +477,7 @@ def main() :
             image = generateBibliometricsImage(
                 metrics,
                 colors,
-                "Bibliometrics"
+                "Bibliometrics",
+                colors["include"] if "include" in colors else stats
             )
             outputImage(image, colors["filename"])
